`timescale 1 ps / 1 ps

module cmdfifo  (
// clock
	clk,
	reset_n,
	
// master
	m_in_data,
	m_in_wr,
	m_out_full,
	m_out_empty,
// slave
	s_out_cnt,
	s_out_data,
	s_in_rd
);

	parameter	Width			= 32;
	parameter	WidthAd			= 5;
	parameter	NumWords		= 32;
	
	// clock
	input					clk;
	input					reset_n;
	
	// master
	input	[Width-1:0]		m_in_data;
	input					m_in_wr;
	output					m_out_empty;
	output					m_out_full;
	
	// slave
	output	[WidthAd:0]		s_out_cnt;
	output	[Width-1:0]		s_out_data;
	input					s_in_rd;
	
	wire	[WidthAd-1:0]	rp;
	wire	[WidthAd-1:0]	wp;
	wire					m_out_empty;
	wire					m_out_full;
	wire	[WidthAd:0]		s_out_cnt;
	wire	[Width-1:0]		s_out_data;
	
	reg	[WidthAd:0]			wcnt;
	reg [WidthAd:0]			rcnt;
	reg [Width-1:0]			data	[NumWords-1:0];

	assign s_out_data		= data[rp];
	assign s_out_cnt		= wcnt-rcnt;
	assign m_out_full		= s_out_cnt[WidthAd];
	assign m_out_empty		= (s_out_cnt == 0);
	assign wp[WidthAd-1:0]	= wcnt[WidthAd-1:0];
	assign rp[WidthAd-1:0]	= rcnt[WidthAd-1:0];
	
	always @(posedge clk or negedge reset_n) begin
		if(!reset_n) begin
			wcnt <= 0;
			rcnt <= 0;
		end
		else begin
			if(m_in_wr & ~m_out_full) begin
				data[wp]	<= m_in_data;
				wcnt		<= wcnt + 1;
			end
			if(s_in_rd & ~m_out_empty) begin
				rcnt		<= rcnt + 1;
			end
		end
	end
	
endmodule

module gfx (
	clk,
	reset_n,
	
	cmd_in_data,
	cmd_in_wr,
	cmd_out_full,
	cmd_out_empty
);

	parameter	Width			= 32;
	parameter	WidthAd			= 5;
	
	// clock
	input					clk;
	input					reset_n;
	
	// master
	input	[Width-1:0]		cmd_in_data;
	input					cmd_in_wr;
	output					cmd_out_empty;
	output					cmd_out_full;

	wire	[Width-1:0]		cmd_in_data;
	wire					cmd_in_wr;
	wire					cmd_out_empty;
	wire					cmd_out_full;

	wire					data_avail;
	wire	[Width-1:0]		read_data;
	wire	[WidthAd:0]		data_cnt;
	reg						rd;
	reg		[Width-1:0]		command;
	reg		[3:0]			params_cnt;	
	reg		[Width-1:0]		params	[0:11];
	reg						exec;
	reg		[31:0]			read_cache;
	reg		[31:0]			write_cache;

	cmdfifo	cmdfifo0(clk, reset_n, cmd_in_data,
		cmd_in_wr, cmd_out_full, cmd_out_empty,
		data_cnt, read_data, rd);
		
	assign data_avail	= ~cmd_out_empty;

	always @(posedge clk) begin
		if(exec) begin
			case (command)
				6'h00:	begin
							$display("NOP");
							exec <= 0;
						end
				6'h01:	begin
							$display("DRAWLINE %x %x %x %x",
								params[0],params[1],params[2],params[3]);
							exec <= 0;
						end
				endcase
		end
	end

	always @(posedge clk or negedge reset_n) begin
		if(!reset_n) begin
			rd <= 1;
			exec <= 0;
			params_cnt <= 0;
		end
		else begin
			if(data_avail & !exec) begin
				
				if(params_cnt == 0) begin
					command <= read_data;
					case (read_data[5:0])
						6'h00:	begin
									params_cnt <= 0;
									exec <= 1;
								end
						6'h01:	begin
									params_cnt <= 4;
								end
					endcase
				end  else begin
					params_cnt <= params_cnt - 1;
					params[params_cnt-1] <= read_data[31:0];
					if(params_cnt == 1) begin
						exec <= 1;
					end
				end
			end
		end
	end
	
endmodule

/*
module gfx
(
// clk:
	clk,
	reset_n,
	
// command fifo
	// in:
	cmdfifo_in_data,
	cmdfifo_in_wr,
	// out:
	cmdfifo_out_full,
	cmdfifi_out_empty,
	
// read mem
	//output:
	readmem_clk,
	readmem_reset_n,
	readmem_out_address,
	readmem_in_readdata,
// write mem
	writemem_clk,
	writemem_reset_n,
	writemem_out_address,
	writemem_out_writedata
);

// parameter
	parameter ADDR_SIZE			32
	parameter FIFO_DATA_SIZE	32
	parameter FIFO_COUNT		32
	parameter FIFO_COUNT_SIZE	6

// port
	input							clk;
	input							reset_n;
	
	input	[FIFO_DATA_SIZE-1:0]	cmdfifo_in_data;
	input								cmdfifo_in_wr;
	output							cmdfifo_out_full,
	output							cmdfifi_out_empty,
	
	output							readmem_out_address;
	output	[31:0]				readmem_in_readdata;

	output							writemem_out_address;
	output	[31:0]				writemem_out_writedata;
	
//------------------------------------------------------------------------------
// Command Queue
//------------------------------------------------------------------------------

'define CMD_SETBUF		4'h0000
'define CMD_CLRSTATE		4'h0001
'define CMD_CLEAR			4'h0002
	
// wire
	// fifo
	wire	[FIFO_DATA_SIZE-1:0]	cmdfifo_out_data;
	wire	[FIFO_COUNT_SIZE-1:0]	cmdfifo_cnt;
	wire	[FIFO_COUNT_SIZE-2:0]	cmdfifo_wp;
	wire	[FIFO_COUNT_SIZE-2:0]	cmdfifo_rp;
// reg
	// fifo
	reg		[FIFO_DATA_SIZE:0]		cmdfifo_data [0:FIFO_COUNT-1];
	reg		[FIFO_COUNT_SIZE-1:0]	cmdfifo_wcnt;
	reg		[FIFO_COUNT_SIZE-1:0]	cmdfifo_rcnt;
// assign
	// fifo
	assign	cmdfifo_out_data = cmdfifo_data[cmdfifo_rp];
	assign	cmdfifo_cnt	= cmdfifo_wcnt - cmdfifo_rcnt;
	assign	cmdfifo_out_full	= cmdfifo_cnt[FIFO_COUNT_SIZE];
	assign	cmdfifo_out_empty	= (cmdfifo_cnt == 0);
	assign	cmdfifo_wp[FIFO_COUNT_SIZE-2] = cmdfifo_wcnt[FIFO_COUNT_SIZE-2:1];
	assign	cmdfifo_rp[FIFO_COUNT_SIZE-2] = cmdfifo_rcnt[FIFO_COUNT_SIZE-2:1];
// always
	// input fifo
	always @(posedge clk or negedge reset_n) begin
		if(!reset_n) begin
			wcnt	<= 0;
			rcnt	<= 0;
		end
		else begin
			if(cmdfifo_in_wr && ~cmdfifo_out_full) begin
				cmdfifo_data[cmdfifo_wp] <= cmdfifo_in_data;
				cmdfifo_wcnt <= cmdfifo_wcnt + 1;
			end
		end
	end
	
//------------------------------------------------------------------------------
// Command Parser
//------------------------------------------------------------------------------

// reg
	reg							cmdparser_active;
	reg	[FIFO_DATA_SIZE-1:0]	cmdparser_command;
	reg [3:0]					cmdparser_prmcnt;

`define CMD_NOP				0
`define CMD_FILLRECT		1
	
	// parse fifo
	always @(posedge clk or negedge reset_n) begin
		if(!reset_n) begin
			cmdparser_active	<= 0;
		end
		else begin
			if(~cmdfifo_out_empty) begin
				if(~cmdparser_active) begin
					cmdparser_command <= cmdfifo_out_data;
					cmdfifo_rcnt <= cmdfifo_rcnt + 1;
					cmdparser_prmcnt <= 0;
					cmdparser_active <= 1;
					
					// パラメータの個数判定
					switch(cmdfifo_out_data) begin
					case `CMD_NOP:
						
					case `CMD_FILLRECT:
					end
				end
			end
		end
	end
endmodule
*/

